open Printf

let rep = 1
let outkind = `Text

let gen_cmd = "./gensplit"

let pgms  = [ 
(*
  (*"reverse.xsltc.xsl";*) "reverse2.xsltc.xsl"; (*"reverse3.xsltc.xsl"; *)

  (*"reverse.saxon.xsl"; "reverse2.saxon.xsl"; "reverse3.saxon.xsl";*)
  "reverse4.saxon.xsl";

  (*"reverse.xsltproc.xsl";*) "reverse2.xsltproc.xsl"; (*"reverse3.xsltproc.xsl";*)
  (*"reverse.xalan.xsl";*) "reverse2.xalan.xsl"; (*"reverse3.xalan.xsl"; *)
*)

  "reverse.stx";
  "reverse.xst";
  "reverse.cd";  
  "reverse.qizx.xq";
  "reverse.saxon.xq";
  "reverse2.xsltc.xsl"; 
  "reverse4.saxon.xsl";
  "reverse2.xsltproc.xsl";
  "reverse2.xalan.xsl";

(*
  "split.xst"; "split2.xst"; "split3.xst"; "split4.xst"; 
  "split.cd"; "split2.cd"; "split3.cd"; 

  "split.saxon.xq"; "split2.saxon.xq"; "split3.saxon.xq"; 
  "split.qizx.xq"; "split2.qizx.xq"; "split3.qizx.xq";

  "split1.xsltc.xsl"; "split2.xsltc.xsl"; "split3.xsltc.xsl"; 
  "split1.saxon.xsl"; "split2.saxon.xsl"; "split3.saxon.xsl"; 
  "split1.xsltproc.xsl"; "split2.xsltproc.xsl"; "split3.xsltproc.xsl";
  "split1.xalan.xsl"; "split2.xalan.xsl";  "split3.xalan.xsl";
  "split.stx"; "split2.stx"; 
*)
]

(*
let pgms = [
  "split4.xst";
  "split2.cd";
  "split2.saxon.xq";
  "split3.qizx.xq";
  "split3.xsltc.xsl";
  "split3.saxon.xsl";
  "split3.xsltproc.xsl";
  "split3.xalan.xsl";
  "split.stx";
]
*)

let sizes = [ 250; 500; 1000; 2000; 4000; 8000; 16000; 32000; 64000 ]
let sizes = [ 2000 ]

exception Error

let cmd s = 
  if (Sys.command s != 0) then raise Error

let gen i =
  let f = sprintf "split%i.xml" i in
  if not (Sys.file_exists f) then (
    eprintf "Generating %s\n" f; flush stderr;
    cmd (sprintf "%s %i > %s" gen_cmd i f)
  );
  let s = (Unix.stat f).Unix.st_size in
  s / 1024 / 1024,
  f

let pr = printf
let gt = Unix.gettimeofday

let measure s =
  let t = gt() in
  cmd (sprintf "%s > /dev/null 2>> log" s);
  gt () -. t

let time i s =
  try
    let a = ref 0. and g = ref 1. in
    for i = 1 to rep do 
      let t = measure s in
      a := !a +. t;
      g := !g *. t;
    done;
    let m = (!g ** (1. /. (float_of_int rep))) in
      (*    (!a /. (float_of_int rep)) *)

    let m = (float_of_int i) /. m in
    printf "%5.2f " m;
    flush stdout
  with Error ->
    printf "*ERR* ";
    flush stdout;
    raise Error
      
let memory s =
  try
    cmd (sprintf "./memusage \"%s > /dev/null 2>> log\"" s);
    flush stdout
  with Error ->
    printf "*ERR* ";
    flush stdout;
    raise Error
      




let xstream pgm =
  cmd (Printf.sprintf "../xstream -concat -o %s.exe %s" pgm pgm);
  fun xml -> Printf.sprintf "./%s.exe %s" pgm xml

let xsltproc pgm xml =
  Printf.sprintf "xsltproc --param input %s %s %s" xml pgm xml

let cduce pgm =
  cmd (Printf.sprintf "~/cduce/cduce -c %s" pgm);
  fun xml -> Printf.sprintf "~/cduce/cduce --run %so --arg %s" pgm xml

let saxon_xq pgm xml =
  Printf.sprintf "./saxon_xquery -pull -s %s %s" xml pgm

let saxon_xsl pgm xml =
  Printf.sprintf "./saxon_xslt %s %s input=%s" xml pgm xml

let qizx pgm xml =
  Printf.sprintf "./qizx -input %s -q %s" xml pgm

let xsltc pgm =
  let classfile = String.sub pgm 0 (String.index pgm '.') in
  cmd (Printf.sprintf "./xsltc_compile -o %s %s" classfile pgm);
  fun xml -> Printf.sprintf "./xsltc_run %s %s input=%s" xml classfile xml
    
let xalan pgm xml =
  Printf.sprintf "xalan -in %s -xsl %s -param input %s" xml pgm xml

let galax pgm xml =
  Printf.sprintf "galax-run -context-item %s %s" xml pgm

let stx pgm xml =
  Printf.sprintf "./stx %s %s" xml pgm

let tools = [ 
  "xsltproc.xsl", "xsltproc (XSLT)", xsltproc;
  "saxon.xsl", "Saxon (XSLT)", saxon_xsl;
  "xalan.xsl", "Xalan (XSLT)", xalan;
  "xsltc.xsl", "XSLTC (XSLT)", xsltc;
  "xst", "XStream", xstream;
  "cd", "CDuce", cduce;
  "saxon.xq", "Saxon (XQuery)", saxon_xq;
  "qizx.xq", "Qizx/open (XQuery)", qizx;
  "galax.xq", "Galax (XQuery)", galax;
  "stx", "Joost (STX)", stx;
]

let xmls = List.map gen sizes

let th () = 
  match outkind with 
    | `Text | `Latex -> () 
    | `Html -> print_string "<th>"
let th_end () = 
  match outkind with 
    | `Text -> ()
    | `Latex -> print_string "&"
    | `Html -> print_string "</th>"
let td () = match outkind with `Text | `Latex -> () | `Html -> print_string "<td>"
let td_end () = 
  match outkind with 
    | `Text -> ()
    | `Latex -> print_string "&"
    | `Html -> print_string "</td>"
let tr () = 
  match outkind with 
    | `Text | `Latex -> () 
    | `Html -> print_string "<tr>"
let tr_end () = 
  match outkind with 
    | `Text -> ()
    | `Latex -> print_string "\\\\"
    | `Html -> print_string "</tr>"

let () =
  let mem = Array.length Sys.argv > 1 && Sys.argv.(1) = "mem" in

  tr ();
  th (); printf "%20s  " ""; th_end ();
  List.iter (fun (i,_) -> th (); printf "%3iMb " i; th_end()) xmls;
  tr_end ();
  printf "\n";

  List.iter 
    (fun pgm ->
       let (_,tool_name, tool) = 
	 try 
	   List.find (fun (ext,_,_) -> Filename.check_suffix pgm ext) tools 
	 with Not_found -> failwith pgm
       in
       let f = tool pgm in
       
       tr ();

       (match outkind with
	  | `Text -> printf "%20s: " pgm
	  | `Latex -> printf "%20s &" tool_name
	  | `Html -> printf "<th>%s <a href=\"benchmarks/%s\">%s</a></th>"
	      tool_name pgm pgm
       ); 
       flush stdout;

       (try 
	 if mem then
	   List.iter (fun (i,xml) -> memory (f xml)) xmls
	 else
	   List.iter (fun (i,xml) -> td (); time i (f xml); td_end ()) xmls;  
       with Error -> ());

       tr_end ();
       printf "\n"
    )
    pgms

  
