module I = struct type t = int let compare = compare end
module IS = Set.Make(I)
module IM = Map.Make(I)

let pids () =
  let r = ref IS.empty in
  Array.iter
    (fun s -> 
       try let i = int_of_string s in r := IS.add i !r with Failure _-> ())
    (Sys.readdir "/proc");
  !r

let maxrss = ref IM.empty
let get pid = try IM.find pid !maxrss with Not_found -> 0
let set pid rss = maxrss := IM.add pid rss !maxrss

let zombie = ref false
let names = ref IM.empty
let themaxrss = ref 0
let themaxname = ref ""

let parse_status pid =
  let rss = ref 0 and name = ref "" in
  begin
    let ic = open_in (Printf.sprintf "/proc/%i/status" pid) in
    try while true do
      let s = input_line ic in
      let r = String.index s ':' in
      let v = String.sub s (r + 2) (String.length s - r - 2) in
      match String.sub s 0 r with
	| "VmRSS" -> Scanf.sscanf v " %i kB" (fun i -> rss := i)
	| "Name" -> name := v
	| "State" -> if v.[0] = 'Z' then zombie := true
	| _ -> ()
    done with End_of_file -> close_in ic
  end;
  set pid (max !rss (get pid));
  names := IM.add pid !name !names;
  if !rss > !themaxrss then (themaxrss := !rss; themaxname := !name)
(*
  Printf.printf "(pid:%i,name:%S,rss:%i)\n" pid !name !rss;
  flush stdout
*)

let run s =
  let p0 = pids () in
  match Unix.fork () with
    | 0 -> ignore (Unix.system s)
    | pid ->
	let f = Printf.sprintf "/proc/%i/status" pid in
(*	prerr_endline f; *)
	while Sys.file_exists f && not !zombie do 
	  Unix.sleep 1; 
	  IS.iter parse_status (IS.diff (pids ()) p0)
	done;
	IS.iter parse_status (IS.diff (pids ()) p0);
(*	IM.iter (fun pid rss -> 
		   Printf.printf "pid:%i,name:%s,rss:%i\n" 
		     pid 
		     (IM.find pid !names)
		     rss)
	  !maxrss; *)
(*	Printf.printf "name:%s;rss:%i\n" !themaxname !themaxrss *)
	Printf.printf "%5.1f " ((float_of_int !themaxrss) /. 1024.)
	   


let () =
  run Sys.argv.(1)
